/**
 * \file clause.h
 * \author Léo Perrin
 * \date 2011/11/22
 */
#ifndef _CLAUSE_H_
#define _CLAUSE_H_

#include <vector>
#include <string>


/**
 * \class Clause
 * \brief This class is used to manipulate CNF clauses.
 * 
 * A CNF clause consist of the disjunction of an arbitrary number of
 * literals. This class makes it easier to use these in coordination
 * with the <a href="http://minisat.se/MiniSat.html">minisat2</a>
 * program.
 */
class Clause
{
	private:
		/**
		 * \brief The codes of the variables of the clause.
		 *
		 * If the literal is negated, it contains the opposite of its
		 * code instead of its code according to the DIMACS encoding.
		 */
		std::vector<int> literals;

	public:
		/**
		 * \brief Builds an empty Clause instance.
		 **/
		Clause();

		/**
		 * \brief Adds the code of the literal given as an
		 * argument to the Clause.
		 */
		void addLit(int);

		/**
		 * \brief Puts all the literals in a one line string.
		 *
		 * The aim of this method is to write a line of a DIMACS
		 * formatted file corresponding to this disjunction.
		 **/
		std::string toString();

		/**
		 * \brief Applies the restriction corresponding to the assign
		 * assignment to the clause.
		 *
		 * For each element of the assign vector, checks if it is in the
		 * variables of this clause. If no literal of the assignment is
		 * in the clause, does nothing. If one of the literals is
		 * present and negated, removes it. If there is no litteral left
		 * in the clause, ends the program: the formula is
		 * unsatisfiable, no need to ask minisat2.
		 * 
		 * \return 1 if one of the literals is present and is not
		 * negated (the clause is true), 0 otherwise. 
		 */
		bool restriction(std::vector<int>);
};

#endif
