/**
 * \file variableset.h
 * \author Léo Perrin
 * \date 2011/11/22
 */

#ifndef _VARIABLES_H_
#define _VARIABLES_H_

#include <vector>


/**
 * \brief Represents a mathematical variable set; is used to associate
 * a unique code to a given number of literals described intuitively
 * by 2 or 3 indices.
 *
 * In order for formula to be written in DIMACS files and to be
 * manipulated by Clause and Formula instance, they must have a unique
 * integer valued code. This class ensures the corresponds between the
 * code and the coordinates of a variable.
 */
class VariableSet
{
	private:
		/**
		 * \brief The dimensions of the matrices to which the
		 * variables correspond. Each vector< unsigned int>
		 * corresponds to one subset of variables.
		 * 
		 * There can be as many dimensions as necessary in
		 * each subset. For instance, if "x" variables are in
		 * a 3x3 matrix and "y" variables in a 2x4 one, then
		 * it should hold that dim = [ [3,3], [2,4] ].
		 */
                std::vector< std::vector<unsigned int> > dim;

                /**
                 * \brief Contains the length of the vector used by
                 * each variables subset. It allows a faster
                 * computation of the index in varValue of a given
                 * literal from its coordinates.
                 */
                std::vector<unsigned int> sizes;

		/**
		 * \brief Contains the value of each variable in the
		 * set; is initialized so as to contain only "false".
		 */
                std::vector<bool> varValue;

	public:

		/**
		 * \brief Constructor for 2D literals (x_ij).
		 *
		 * Initializes the VariableSet instance by creating a
		 * varValue vector attribute of the correct size and
		 * filling it with zeros.
		 *
		 * \param i First dimension describing the literals.
		 * \param j The second dimension.
		 */
		VariableSet(unsigned int,unsigned int);

		/**
		 * \brief Constructor for 3D literals (x_ijk).
		 *
		 * Initializes the VariableSet instance by creating a
		 * varValue vector attribute of the correct size and
		 * filling it with zeros.
		 * 
		 * \param i First dimension describing the literals.
		 * \param j The second dimension.
		 * \param k The third one.
		 */
		VariableSet(unsigned int,unsigned int,unsigned int);

                /**
                 * \brief Adds a new subset of variables to this
                 * instance.
                 *
                 * Their coordinates are indepedents from any other
                 * subset.
                 *
                 * \param i First coordinate.
                 * \param j First coordinate.
                 */
                void addVars(unsigned int, unsigned int);

                /**
                 * \brief Adds a new subset of variables to this
                 * instance.
                 *
                 * Their coordinates are indepedents from any other
                 * subset.
                 *
                 * \param i First coordinate.
                 * \param j First coordinate.
                 * \param k First coordinate.
                 */
                void addVars(unsigned int, unsigned int, unsigned int);

		/**
		 * \brief Returns the code of the 2D variable designed
		 * by the coordinates given as argument in the l_th
		 * subset.
		 *
                 * \param s subset index
		 * \param i First coordinate.
		 * \param j Second coordinate.
		 * 
		 * \return The code of x_ij
		 */
		 unsigned int getVarCode(unsigned int,unsigned int,unsigned int);

		/**
		 * \brief Returns the code of the 3D variable designed
		 * by the coordinates given as argument.
		 *
                 * \param s subset index
		 * \param i First coordinate.
		 * \param j Second coordinate.
		 * \param k Third one.
		 *
		 * \return The code of x_ijk
		 */
		 unsigned int getVarCode(unsigned int,unsigned int,unsigned int,unsigned int);

		/**
		 * \brief Returns the value of the variable described
		 * by the given 2D coordinates.
		 *
                 * \param s subset index
		 * \param i First coordinate.
		 * \param j Second coordinate.
		 *
		 * \return The value of x_ij.
		 */
		bool getVarValue(unsigned int,unsigned int,unsigned int);
		
		/**
		 * \brief Returns the value of the variable described
		 * by the given 3D coordinates.
		 *
                 * \param s subset index
		 * \param i First coordinate.
		 * \param j Second coordinate.
		 * \param k Third one.
		 *
		 * \return The value of x_ijk.
		 */
		bool getVarValue(unsigned int,unsigned int,unsigned int,unsigned int);


		/**
		 * \brief Returns the total number of variables in
		 * this set.
		 *
		 * \return The cardinal of the set of the variables.
		 */
		 unsigned int getCard();

		/**
		 * \brief Parses a dimacs results formatted stream and
		 * assigns its variables accordingly.
		 *
		 * Any previous assignment is erased, in particular
		 * that of the construction.
		 *
		 * \param input The input stream to the file to parse.
		 */
		void parseDimacs(std::istream *);

};

#endif
